//
//  ChunksDisplay.java
//  CocoDonkey
//  $Id: ChunksDisplay.java,v 1.4 2003/01/01 19:26:33 fortun Exp $
//
//  Created by Fred Bonzoun on Sun May 12 2002.
//  Copyright (c) 2002 Bonzoun. All rights reserved.
//
//  This library is free software; you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published
//  by the Free Software Foundation; either version 2.1 of the License, or
//  (at your option) any later version.
//
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

import com.apple.cocoa.foundation.*;
import com.apple.cocoa.application.*;


public class ChunksDisplay extends NSView {

    private String chunks;
    
    public ChunksDisplay(NSRect frame) {
        super(frame);
        // Initialization code here.        
    }

    public void setChunks(String chunks) {
        this.chunks = chunks;
        setNeedsDisplay(true);
    }
    
    public void drawRect(NSRect rect) {
        // Drawing code here.
        if (chunks!=null) {
            float w = frame().width() / chunks.length();
            float h = frame().height();
            NSColor color = null;
            char oldChunk = chunks.charAt(0);
            int oldI = 0;
            for(int i=0; i<chunks.length(); i++) {
                char c = chunks.charAt(i);
                if (oldChunk != c) {
                    oldChunk = c;
                    color.set();
                    NSBezierPath.fillRect(new NSRect(w*oldI, 0, w*(i-oldI), h));
                    oldI = i;
                }
                switch(c) {
                    case 'V':
                    case '1':
                        color = NSColor.colorWithDeviceRGB(0.0f, 1.0f, 0.0f, 1f);
                        break;
                    case 'p':
                        color = NSColor.colorWithDeviceRGB(0.5f, 1.0f, 0.5f, 1f);
                        break;
                    case '!':
                        color = NSColor.colorWithDeviceRGB(1.0f, 1.0f, 0.0f, 1f);
                        break;
                    case '?':
                        color = NSColor.colorWithDeviceRGB(0.8f, 0.8f, 0.0f, 1f);
                        break;
                    case '_':
                    case '0':
                        color = NSColor.colorWithDeviceRGB(1.0f, 0.0f, 0.0f, 1f);
                        break;
                    case '.':
                        color = NSColor.colorWithDeviceRGB(1.0f, 0.5f, 0.5f, 1f);
                        break;
                    default:
                        color = NSColor.lightGrayColor();
                        break;
                }
            }
            color.set();
            NSBezierPath.fillRect(new NSRect(w*oldI, 0, w*(chunks.length()-oldI), h));
        } else {
            NSColor.lightGrayColor().set();
            NSBezierPath.fillRect(frame());
        }
    }
    
}

// $Log: ChunksDisplay.java,v $
// Revision 1.4  2003/01/01 19:26:33  fortun
// Compatibility with 2.00
//
// Revision 1.3  2002/08/21 21:43:14  fortun
// Optimized display
//
// Revision 1.2  2002/06/29 11:19:47  fortun
// More colors for the cached chunk types
//
// Revision 1.1.1.1  2002/05/21 21:26:10  fortun
//
//
