//
//  DownloadItemSource.java
//  CocoDonkey
//  $Id: DownloadItemSource.java,v 1.16 2003/01/02 21:31:43 fortun Exp $
//
//  Created by Fred Bonzoun on Sat May 11 2002.
//  Copyright (c) 2002 Bonzoun. All rights reserved.
//
//  This library is free software; you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published
//  by the Free Software Foundation; either version 2.1 of the License, or
//  (at your option) any later version.
//
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

import com.apple.cocoa.foundation.*;
import com.apple.cocoa.application.*;

import net.bonzoun.cocodonkey.*;
import java.util.*;

/***
 * Source for a TableView containing a list of the currently downloading files
 ***/
public class DownloadItemSource {

    NSTableView tableView;
    TableViewSorter tableSorter;
    
    private HashMap items = new HashMap();

    public void windowDidLoad() {
        tableSorter.addSortedColumn("Name", new DownloadItem.CompareName());
        tableSorter.addSortedColumn("Size", new DownloadItem.CompareSize());
        tableSorter.addSortedColumn("Download", new DownloadItem.CompareDownloaded());
        tableSorter.addSortedColumn("Percent", new DownloadItem.ComparePercent());
        tableSorter.addSortedColumn("Rate", new DownloadItem.CompareRate());
        tableView.tableColumnWithIdentifier("Percent").setDataCell(new PercentDisplay());
    }

    public int itemsCount() {
        return items.size();
    }
    
    /*package*/ void clean() {
        items = new HashMap();
        tableSorter.setList(new ArrayList());
        if (tableView.selectedRow()>=0)
            tableView.deselectRow(tableView.selectedRow());
        //tableView.setNeedsDisplay(true); // Done in TableSorter
    }
    
    /*package*/ DownloadItem itemAtRow(int row) {
        if (row<items.size())
            return (DownloadItem)tableSorter.list().get(row);
        else
            return null;
    }

    /*package*/ DownloadItem item(int size, String hash) {
        List list = tableSorter.list();
        for(int i=0; i<list.size(); i++) {
            DownloadItem item = (DownloadItem)list.get(i);
            if (item.size()==size && item.hash()!=null && item.hash().equals(hash))
                return item;
        }
        return null;
    }

    /*package*/ DownloadItem item(String name) {
        List list = tableSorter.list();
        for(int i=0; i<list.size(); i++) {
            DownloadItem item = (DownloadItem)list.get(i);
            if (item.name()!=null && item.name().equals(name))
                return item;
        }
        return null;
    }
    
    /*package*/ float totalRate() {
        float total = 0;
        for(int i=0; i<tableSorter.list().size(); i++) {
            DownloadItem item = (DownloadItem)tableSorter.list().get(i);
            if (item.rate()>0)
                total += item.rate();
        }
        return total;
    }
    
    public void addElements(DonkeyTalker talker, NSArray liste) {
        for(int i=0; i<liste.count(); i++) {
            DownloadItem item = (DownloadItem)liste.objectAtIndex(i);
            Number id = new Integer(item.id());
            DownloadItem orig = (DownloadItem)items.get(id);
            if (orig==null) {
                //if (item.hash()==null)
                //    talker.fillItemDetails(item);
                if (tableView.canDraw())
                    tableView.lockFocus();
                tableSorter.addObject(item);
                items.put(id, item);
                //int n = tableSorter.list().indexOf(item);
                //tableView.setNeedsDisplay(tableView.rectOfRow(n)); // Done in the sorter
                //tableView.reloadData();
                if (tableView.canDraw())
                    tableView.unlockFocus();
            } else {
                if (orig.hash()==null || orig.updateWith(item)) {
                    if (orig.hash()==null)
                        talker.fillItemDetails(orig, false);
                    if (tableView.canDraw())
                        tableView.lockFocus();
                    int n = tableSorter.list().indexOf(orig);
                    tableView.setNeedsDisplay(tableView.rectOfRow(n));
                    if (tableView.canDraw())
                        tableView.unlockFocus();
                }
            }
        }
    }
    
    public int numberOfRowsInTableView(NSTableView aTableView) {
        return items.size();
    }

    public final Object tableViewObjectValueForLocation(NSTableView aTableView, NSTableColumn aTableColumn, int rowIndex) {
        if (rowIndex>=tableSorter.list().size())
            return null;
        DownloadItem item = (DownloadItem)tableSorter.list().get(rowIndex);
        String name = (String)aTableColumn.identifier();
        if (name.equals("Name"))
            return item.name();
        if (name.equals("Percent"))
            return new Double(item.percent()); //String.valueOf(item.percent()) + "%";
        if (name.equals("Size"))
            return item.prettySize();
        if (name.equals("Download"))
            return item.prettyDownloaded();
        if (name.equals("Rate")) {
            if (item.rate()>0)
                return new Float(item.rate());
            else if (item.isFinished())
                return Messages.string("Finished");
            else if (item.rate()==-2)
                return Messages.string("Paused");
            else
                return Messages.string("Stalled");
        }
        if (name.equals("Remaining")) {
            if (item.isFinished())
                return Messages.string("Finished");
            else if (item.rate()==-2)
                return Messages.string("Paused");
            else
                return item.displayTime(item.timeLeft());
        }
        
        return "?";
    }

    /** Sorts the list **/
    public void tableViewDidClickTableColumn(NSTableView tableView, NSTableColumn tableColumn) {
        tableSorter.tableViewDidClickTableColumn(tableView, tableColumn);
    }
}

// $Log: DownloadItemSource.java,v $
// Revision 1.16  2003/01/02 21:31:43  fortun
// Bug in the lockFocus() method if called when drawing is not possible
//
// Revision 1.15  2003/01/01 22:47:05  fortun
// Acceleration of the reading of the hash value
//
// Revision 1.14  2003/01/01 19:28:05  fortun
// Delay in the full update of the screen
//
// Revision 1.13  2002/07/30 00:13:40  fortun
// Reordering of the code in the addElements() method
//
// Revision 1.12  2002/07/28 18:48:32  fortun
// Optimization: selection emptied when list emptied
//
// Revision 1.11  2002/07/22 00:30:23  fortun
// NSArrays are now Lists
//
// Revision 1.10  2002/07/16 00:13:11  fortun
// Percent is now displayed graphically
//
// Revision 1.9  2002/07/07 22:40:22  fortun
// Simplification
//
// Revision 1.8  2002/07/07 20:07:28  fortun
// Simpler handling of the selection of the header
//
// Revision 1.7  2002/07/07 19:39:04  fortun
// Now handles the selection of the header
// Empty trash does not delete anything now
//
// Revision 1.6  2002/06/30 21:01:26  fortun
// Better GUI locks when getting new elements
//
// Revision 1.5  2002/06/29 17:52:59  fortun
// Added some locks to prevent problems with the GUI
//
// Revision 1.4  2002/06/09 15:38:36  fortun
// Better info in the "Remaining" column
//
// Revision 1.3  2002/06/08 12:16:34  fortun
// Display remaining time
//
// Revision 1.2  2002/05/27 22:01:45  fortun
// Uses the new TableViewSorter to sort the list
//
// Revision 1.1.1.1  2002/05/21 21:26:10  fortun
//
//
