//
//  MainMenuController.java
//  CocoDonkey
//  $Id: MainMenuController.java,v 1.8 2002/08/21 21:41:44 fortun Exp $
//
//  Created by Fred Bonzoun on Sun May 12 2002.
//  Copyright (c) 2002 Bonzoun. All rights reserved.
//
//  This library is free software; you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published
//  by the Free Software Foundation; either version 2.1 of the License, or
//  (at your option) any later version.
//
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

import com.apple.cocoa.foundation.*;
import com.apple.cocoa.application.*;

import net.bonzoun.cocodonkey.*;

public class MainMenuController {

    NSMenu mainMenu;

    public void applicationDidFinishLaunching(NSNotification aNotification) {
        NSSelector action = new NSSelector("openRecentConnection", new Class[]{Object.class});
        // We get the right menu
        NSMenuItem fileItem = mainMenu.itemWithTag(1);
        NSMenuItem recentItem = fileItem.submenu().itemWithTag(1);
        NSMenu recentMenu = recentItem.submenu();
        // We insert the recent connections
        NSArray list = Preferences.recentConnections();
        for(int i=list.count()-1; i>=0; i--) {
            NSDictionary dico = (NSDictionary)list.objectAtIndex(i);
            String name = (String)dico.objectForKey("name");
            if (name==null)
                name = dico.objectForKey("hostname") + " - " + dico.objectForKey("type");
            NSMenuItem item = new NSMenuItem(name, action, "");
            item.setTarget(this);
            item.setRepresentedObject(dico);
            recentMenu.insertItemAtIndex(item, 0);
        }
        if (Preferences.autoConnectDaemon())
            openDonkey(this);
    }
    
    public void openDonkey(Object sender) {
        MainWindowController controller = new MainWindowController(new DonkeyTalker(Preferences.daemonLocation(), new Logger()));
        controller.window().setTitle(Messages.string("Local Daemon"));
        controller.showWindow(controller.window());
    }

    public void newConnection(Object sender) {
        NewConnectionPanelController controller = new NewConnectionPanelController();
        NSApplication.sharedApplication().runModalForWindow(controller.window());
        DonkeyTalker talker = controller.talker();
        if (talker!=null) {
            MainWindowController mcontroller = new MainWindowController(talker);
            mcontroller.window().setTitle(controller.hostname());
            mcontroller.showWindow(mcontroller.window());
        }
    }

    public void openRecentConnection(Object sender) {
        NSMenuItem item = (NSMenuItem)sender;
        NSDictionary connDict = (NSDictionary)item.representedObject();
        String type = (String)connDict.objectForKey("type");
        if (type!=null) {
            DonkeyTalker talker = null;
            if (type.equals("ssh")) {
                talker = NewConnectionPanelController.openSshConnection((String)connDict.objectForKey("hostname"));
            }
            else if (type.equals("telnet")) {
                talker = new DonkeyTalker(Preferences.daemonLocation(), (String)connDict.objectForKey("hostname"), new Logger());
            }
            else if (type.equals("local")) {
                talker = new DonkeyTalker(Preferences.daemonLocation(), new Logger());
            }
            if (talker!=null) {
                MainWindowController mcontroller = new MainWindowController(talker);
                mcontroller.window().setTitle((String)connDict.objectForKey("name"));
                mcontroller.showWindow(mcontroller.window());
            }
        }
    }
    
	public void preferences(Object sender) {
        PreferencesPanelController prefs = new PreferencesPanelController();
        NSApplication.sharedApplication().runModalForWindow(prefs.window());
    }

    public boolean applicationShouldTerminate(NSApplication sender) {
        NSArray windows = sender.windows();
        for(int i=0; i<windows.count(); i++) {
            NSWindow w = (NSWindow)windows.objectAtIndex(i);
            if (w.isVisible()) {
                w.performClose(this);
                if (w.isVisible())
                    return false;
            }
        }
        WorkerThread.prepareForExit();
        return true;
    }

    public void showHelp(Object sender) {
        NSWindowController controller = new WindowController("HelpWindow", false);
        controller.showWindow(controller.window());
    }
}

// $Log: MainMenuController.java,v $
// Revision 1.8  2002/08/21 21:41:44  fortun
// new Talker creation syntax
//
// Revision 1.7  2002/07/05 22:46:25  uid85676
// New showHelp() action
//
// Revision 1.6  2002/07/04 23:53:45  fortun
// New Logger parameter to launch the core
//
// Revision 1.5  2002/06/30 21:02:05  fortun
// New autoconnect option handled
//
// Revision 1.4  2002/06/02 17:50:28  fortun
// RecentItems implemented
// Properly stops the threads before quitting
//
// Revision 1.3  2002/06/02 11:56:15  fortun
// Intercepts the applicationShouldTerminate event to properly close all the windows
//
// Revision 1.2  2002/05/25 23:10:38  fortun
// New "newConnection" menu
//
// Revision 1.1.1.1  2002/05/21 21:26:10  fortun
//
//
