//
//  MainWindowController.java
//  CocoDonkey
//  $Id: MainWindowController.java,v 1.25 2003/01/01 22:47:05 fortun Exp $
//
//  Created by Fred Bonzoun on Sun May 12 2002.
//  Copyright (c) 2002 Bonzoun. All rights reserved.
//
//  This library is free software; you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published
//  by the Free Software Foundation; either version 2.1 of the License, or
//  (at your option) any later version.
//
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

import com.apple.cocoa.foundation.*;
import com.apple.cocoa.application.*;

import java.io.*;
import net.bonzoun.cocodonkey.*;

public class MainWindowController extends WindowController {

    ChunksDisplay chunksView; /* IBOutlet */
    DownloadItemSource dataSource; /* IBOutlet */
    NSTableView downloadList; /* IBOutlet */
    NSTextField downSpeedField; /* IBOutlet */
    NSTextField fileNameField; /* IBOutlet */
    NSTextField identifierField; /* IBOutlet */
    NSDrawer infoView; /* IBOutlet */
    NSTextField nbClientsField; /* IBOutlet */
    NSTextField upSpeedField; /* IBOutlet */
    NSTextField chunksCommentField; /* IBOutlet */
    NSTextField ed2kField; /* IBOutlet */

    private Worker thread;
    private DonkeyTalker talker;
    private NSNumberFormatter numberFormatter;
    private UploadSpeed uploadSpeed = new UploadSpeed();
    private DaemonOptionsPanelController daemonOptionsPanelController;
    
    public MainWindowController(DonkeyTalker talker) {
        super("MainWindow");
        this.talker = talker;
    }
    
    public void windowDidLoad() {
        super.windowDidLoad();
        if (Preferences.rememberWindowPos())
            Preferences.restoreColumnSizes(this, downloadList);
        // window();
        //infoView.setContentSize(new NSSize(600, 153));
        numberFormatter = new NSNumberFormatter();
        numberFormatter.setFormat("0.0");
        window().setToolbar(new MainWindowToolbar(this));

        while(true) {
            try {
                talker.connect();
                break;
            }
            catch(IOException e) {
                if (Preferences.autoLaunchDaemon()
                    || NSAlertPanel.runInformationalAlert(
                        Messages.string("No local Donkey running"),
                        Messages.string("Do you want to launch the local Donkey ?\n\nProblem is: %@", e.getMessage()),
                        Messages.string("Yes"), Messages.string("No"), null)
                    == NSAlertPanel.DefaultReturn) {
                    try {
                        DonkeyLauncher.launchLocalDonkey(Preferences.daemonLocation(), talker);
                    }
                    catch(IOException e2) {
                        NSAlertPanel.runCriticalAlert(Messages.string("Unable to launch local Donkey"),
                                                      e2.getMessage(),
                                                      Messages.string("OK"), null, null);
                        window().close();
                        return;
                    }
                } else {
                    window().close();
                    return;
                }
            }
        }
        dataSource.windowDidLoad();
        thread = new Worker();
        thread.start();
    }

    public boolean processIsLocal() {
        return talker.processIsLocal();
    }
    
    public void startSearch(Object sender) {
        SearchWindowController controller = new SearchWindowController(talker, dataSource);
        controller.window().setTitle(controller.window().title() + " - " + window().title());
        controller.showWindow(controller.window());
    }

    public void ed2kLink(Object sender) {
        Ed2kLinkPanelController controller = new Ed2kLinkPanelController(talker);
        controller.window().setTitle(controller.window().title() + " - " + window().title());
        controller.showWindow(controller.window());
    }
    
    public void openOptions(Object sender) {
        if (daemonOptionsPanelController==null || daemonOptionsPanelController.window()==null || !daemonOptionsPanelController.window().isVisible()) {
            daemonOptionsPanelController = new DaemonOptionsPanelController(talker, dataSource);
            daemonOptionsPanelController.window().setTitle(daemonOptionsPanelController.window().title() + " - " + window().title());
        }
        daemonOptionsPanelController.showWindow(daemonOptionsPanelController.window());
        // NSApplication.sharedApplication().runModalForWindow(controller.window());
    }
    
    public void cancelDownload(Object sender) {
        int row = downloadList.selectedRow();
        if (row>=0) {
            if (NSAlertPanel.runInformationalAlert(Messages.string("Do you confirm the cancel ?"),
                                                   Messages.string("The cancel of the download will lose any download already done on the file selected"),
                                                   Messages.string("Yes"), Messages.string("No"), null)
                == NSAlertPanel.DefaultReturn) {
                DownloadItem item = dataSource.itemAtRow(row);
                talker.cancel(item);
                hideInfos();
                dataSource.clean();
            }
        }
    }

    public void resumeDownload(Object sender) {
        int row = downloadList.selectedRow();
        if (row>=0) {
            DownloadItem item = dataSource.itemAtRow(row);
            talker.resume(item);
            downloadList.setNeedsDisplay(downloadList.rectOfRow(row));
        }
    }

    public void pauseDownload(Object sender) {
        int row = downloadList.selectedRow();
        if (row>=0) {
            DownloadItem item = dataSource.itemAtRow(row);
            talker.pause(item);
            downloadList.setNeedsDisplay(downloadList.rectOfRow(row));
        }
    }
    
    public void commit(Object sender) {
        talker.commit();
        hideInfos();
        dataSource.clean();
        if (Preferences.convertAvi() && AviConverterWindowController.nextFileToConvert()!=null) {
            if (NSAlertPanel.runInformationalAlert(
                    Messages.string("Avi files have been commited"),
                    Messages.string("Do you want to translate them into Quicktime format ?"),
                    Messages.string("Yes"), Messages.string("No"), null)
                == NSAlertPanel.DefaultReturn) {
                AviConverterWindowController controller = new AviConverterWindowController();
                controller.window().setTitle(controller.window().title() + " - " + window().title());
            }
        }
    }

    public void openIncoming(Object sender) {
        try {
            Runtime.getRuntime().exec(new String[]{"open", Preferences.daemonLocation() + "/incoming"});
        }
        catch(IOException e) {
            e.printStackTrace();
        }
    }
    
    public void preview(Object sender) {
        int row = downloadList.selectedRow();
        if (row>=0) {
            DownloadItem item = dataSource.itemAtRow(row);
            PreviewWindowController controller = new PreviewWindowController(item, talker, new File("/usr/local/bin/mplayer").exists());
            controller.window().setTitle(controller.window().title() + " - " + window().title());
            controller.showWindow(controller.window());
        }
    }
    
	public boolean windowShouldClose(NSNotification aNotification) {
        hideInfos();
        if (talker.processIsLocal()) {
            int cr;
            if (Preferences.autoStopDaemon())
                cr = NSAlertPanel.AlternateReturn;
            else
                cr = NSAlertPanel.runInformationalAlert(Messages.string("Local Donkey still running"),
                                Messages.string("Do you want to stop the local Donkey ?"),
                                Messages.string("No"), Messages.string("Yes"), Messages.string("Cancel"));
            if (cr == NSAlertPanel.AlternateReturn) {
                if (!talker.finalKill()) {
                    cr = NSAlertPanel.runInformationalAlert(Messages.string("Local Donkey still running"),
                                Messages.string("Do you want to kill the local daemon ?"),
                                Messages.string("No"), Messages.string("Yes"), null);
                    if (cr == NSAlertPanel.AlternateReturn && !DonkeyLauncher.hardKillCore()) {
                        NSAlertPanel.runInformationalAlert(Messages.string("Local Donkey still running"),
                                                       Messages.string("Could not stop the local donkey"),
                                                       "OK", null, null);
                    }
                }
            }
            else if (cr== NSAlertPanel.OtherReturn) {
                return false;
            } else {
                talker.quit();
            }
        } else {
			talker.quit();
		}
        return true;
    }

    public void windowWillClose(NSNotification aNotification) {
        super.windowWillClose(aNotification);
        if (thread!=null) {
            thread.stopRunning();
            thread = null;
        }
        Preferences.rememberColumnSizes(this, downloadList);
        daemonOptionsPanelController = null;
        ((MainWindowToolbar)window().toolbar()).willClose();
    }
    
    public void tableViewSelectionDidChange(NSNotification aNotification) {
        int row = downloadList.selectedRow();
        if (row>=0) {
            fillDrawerInfo();
            showInfos();
        } else {
            hideInfos();
        }
    }

    public void tableViewDidClickTableColumn(NSTableView tableView, NSTableColumn tableColumn) {
        dataSource.tableViewDidClickTableColumn(tableView, tableColumn);
        fillDrawerInfo();
    }

    protected NSColor colorForRow(NSTableView aTableView, int rowIndex) {
        DownloadItem item = dataSource.itemAtRow(rowIndex);
        if (item.isFinished())
            return NSColor.colorWithDeviceRGB(0.92f, 1.0f, 0.92f, 1.0f);
        else if (item.percent()==0)
            return NSColor.colorWithDeviceRGB(1.0f, 0.95f, 0.95f, 1.0f);
        else if (item.rate()==-1)
            return NSColor.colorWithDeviceRGB(1.0f, 1.0f, 0.90f, 1.0f);
        else
            return null;
    }
    
    private void fillDrawerInfo() {
        int row = downloadList.selectedRow();
        if (row>=0) {
            DownloadItem item = dataSource.itemAtRow(row);
            if (item!=null) {
                talker.fillItemDetails(item, true);
                fileNameField.setStringValue(item.name());
                identifierField.setStringValue(item.size() + " " + item.hash());
                ed2kField.setStringValue("ed2k://|file|" + item.name() + "|" + item.size() + "|" + item.hash() + "|");
                chunksView.setChunks(item.chunks());
                nbClientsField.setStringValue("" + item.nbConn() + "/" + item.nbClients());
                int nbFound = (int)(item.chunksFoundPercent()+0.5);
                int nbNotFound = (int)(item.chunksNotFoundPercent()+0.5);
                chunksCommentField.setStringValue(Messages.string("%@% of the chunks found, %@% not found anywhere", new Integer(nbFound), new Integer(nbNotFound)));
            } else {
                hideInfos();
            }
        } else {
            hideInfos();
        }
    }
    
    public void toggleInfoPanel(Object sender) {
        if (downloadList.selectedRow()>=0 || infoView.state()==NSDrawer.OpenState)
            infoView.toggle(sender);
    }
    
    private void hideInfos() {
        infoView.close();
    }

    private void showInfos() {
        infoView.open();
    }

    public boolean isDrawerOpened() {
        return infoView.state()==NSDrawer.OpenState;
    }
    
    public boolean hasSelectedItem() {
        return downloadList.selectedRow()>=0;
    }

    public boolean hasFinishedItems() {
        for(int i=0; i<dataSource.itemsCount(); i++) {
            if (dataSource.itemAtRow(i).isFinished())
                return true;
        }
        return false;
    }
    
	private class Worker extends WorkerThread {

        public void update(NSTimer timer) {
            NSArray liste = (NSArray)timer.userInfo();
            dataSource.addElements(talker, liste);
        }
        
        protected boolean work() throws Exception {
            // doTask("update", talker.downloadItems());
            dataSource.addElements(talker, talker.downloadItems());
            downSpeedField.setStringValue(numberFormatter.stringForObjectValue(new Double(dataSource.totalRate())) + " kB/s");
            if (infoView.state()==NSDrawer.OpenState)
                fillDrawerInfo();
            talker.uploadStats(uploadSpeed);
            double speed = uploadSpeed.speed();
            if (speed==-1)
                upSpeedField.setStringValue("-");
            else
                upSpeedField.setStringValue(numberFormatter.stringForObjectValue(new Double(speed)) + " kB/s");
            return true;
        }

        protected void dies() throws Exception {
            talker.quit();
        }
	}
}

// $Log: MainWindowController.java,v $
// Revision 1.25  2003/01/01 22:47:05  fortun
// Acceleration of the reading of the hash value
//
// Revision 1.24  2002/07/30 00:15:07  fortun
// Little change in the colors for the lines
//
// Revision 1.23  2002/07/28 19:45:48  fortun
// New processIsLocal() method
//
// Revision 1.22  2002/07/21 18:34:14  fortun
// colors changed. Drawer now has a good size
//
// Revision 1.21  2002/07/21 15:14:25  fortun
// Rows get a color depending on their status
//
// Revision 1.20  2002/07/14 22:36:23  fortun
// Some instance variables are cleaned when closing to ease the gc task
//
// Revision 1.19  2002/07/08 21:44:56  fortun
// Drawer is closed before the window, otherwise the application won't quit
//
// Revision 1.18  2002/07/07 22:42:08  fortun
// Killing the core is now a question
// Update of the drawer when sorting
//
// Revision 1.17  2002/07/07 20:07:28  fortun
// Simpler handling of the selection of the header
//
// Revision 1.16  2002/07/07 19:38:45  fortun
// Now handles the selection of the header
//
// Revision 1.15  2002/07/07 01:19:55  fortun
// Displays the ed2k url
// Uses mplayer for the preview
//
// Revision 1.14  2002/07/04 23:54:14  fortun
// New Logger parameter to launch the core
//
// Revision 1.13  2002/07/04 23:14:28  fortun
// The local core is now hard killed if the soft kill fails
//
// Revision 1.12  2002/06/30 21:03:54  fortun
// Remembers the window pos & size
// New autoconnect option handled
// New convertAvi option handled
// New autostop option handled
// GUI update is now handled back in the thread, to get better GUI experience
//
// Revision 1.11  2002/06/29 17:53:34  fortun
// When quitting and using a tunnel, thquit() is now properly sent
//
// Revision 1.10  2002/06/29 11:14:08  fortun
// finalClose() is now called in the quit() method
//
// Revision 1.9  2002/06/14 00:36:49  fortun
// Better close of the connection when quitting
//
// Revision 1.8  2002/06/09 15:39:26  fortun
// Added a "chunks Comment" field
//
// Revision 1.7  2002/06/08 12:17:41  fortun
// hideInfos() calls added
//
// Revision 1.6  2002/06/02 17:51:27  fortun
// When closing the window, quits the connection to the daemon
//
// Revision 1.5  2002/06/02 11:57:48  fortun
// Allows the user to cancel the close of the window
//
// Revision 1.4  2002/06/01 15:15:02  fortun
// Better stop of the WorkerThread
//
// Revision 1.3  2002/05/27 22:04:47  fortun
// Delegates sorting to the dataSource
//
// Revision 1.2  2002/05/25 23:15:11  fortun
// Only one options panel at a time
// Asks for a confirmation on cancel for the downloads
// Tries more heavily to kill the local donkey when asked for
//
// Revision 1.1.1.1  2002/05/21 21:26:10  fortun
//
//
