//
//  NewConnectionPanelController.java
//  CocoDonkey
//  $Id: NewConnectionPanelController.java,v 1.5 2002/08/21 21:42:26 fortun Exp $
//
//  Created by Fred Bonzoun on Thu May 21 2002.
//  Copyright (c) 2002 Bonzoun. All rights reserved.
//
//  This library is free software; you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published
//  by the Free Software Foundation; either version 2.1 of the License, or
//  (at your option) any later version.
//
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

import com.apple.cocoa.foundation.*;
import com.apple.cocoa.application.*;

import java.io.*;
import net.bonzoun.cocodonkey.*;

public class NewConnectionPanelController extends WindowController {

    NSMatrix accessMatrix; /* IBOutlet */
    NSTextField hostnameField; /* IBOutlet */

    private DonkeyTalker talker;
    private String hostname;

    public static DonkeyTalker openSshConnection(String hostname) {
        // SSH Connection
        try {
            return new DonkeyTalker(Preferences.daemonLocation(), new SshTunnel(hostname), new Logger());
        }
        catch(IOException e) {
            NSAlertPanel.runInformationalAlert(Messages.string("Unable to reach host"),
                                               Messages.string("Problem is: %@", e.getMessage()),
                                               Messages.string("OK"), null, null);
            return null;
        }
    }
    
    public NewConnectionPanelController() {
        super("NewConnectionPanel");
        //System.loadLibrary("FileLauncher");
    }
    
    public void cancel(Object sender) { // IBAction
        NSApplication.sharedApplication().stopModalWithCode(0);
        window().close();
    }

    public void ok(Object sender) { // IBAction
        NSCell cell = accessMatrix.selectedCell();
        NSMutableDictionary connDict = new NSMutableDictionary();
        if (cell!=null && cell.tag()==1) {
            // SSH Connection
            talker = openSshConnection(hostnameField.stringValue());
            if (talker==null)
                return;
            hostname = hostnameField.stringValue() + " (SSH)";
            connDict.setObjectForKey("ssh", "type");
            connDict.setObjectForKey(hostnameField.stringValue(), "hostname");
        } else if (cell!=null && cell.tag()==2) {
            // Direct Connection
            talker = new DonkeyTalker(Preferences.daemonLocation(), hostnameField.stringValue(), new Logger());
            hostname = hostnameField.stringValue();
            connDict.setObjectForKey("telnet", "type");
            connDict.setObjectForKey(hostnameField.stringValue(), "hostname");
        } else {
            // Local Connection
            talker = new DonkeyTalker(Preferences.daemonLocation(), new Logger());
            hostname = Messages.string("Local Daemon");
            connDict.setObjectForKey("local", "type");
        }
        connDict.setObjectForKey(hostname, "name");
        Preferences.addRecentConnection(connDict);
        NSApplication.sharedApplication().stopModalWithCode(1);
        window().close();
    }
    
    public void accessMethodChanged(Object sender) {
        NSCell cell = accessMatrix.selectedCell();
        if (cell==null || cell.tag()==3) {
            hostnameField.setEnabled(false);
        } else {
            hostnameField.setEnabled(true);
            window().makeFirstResponder(hostnameField);
        }
    }
    
    public DonkeyTalker talker() {
        return talker;
    }

    public String hostname() {
        return hostname;
    }
}
