//
//  PercentDisplay.java
//  CocoDonkey
//	$Id*
//
//  Created by Fred Bonzoun on Mon Jul 15 2002.
//  Copyright (c) 2002 Bonzoun. All rights reserved.
//
//  This library is free software; you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published
//  by the Free Software Foundation; either version 2.1 of the License, or
//  (at your option) any later version.
//
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

import com.apple.cocoa.foundation.*;
import com.apple.cocoa.application.*;


public class PercentDisplay extends NSCell {

    private static NSImage barBorder;
    private static NSRect barBorderRect;
    private static NSImage barEmpty;
    private static NSRect barEmptyRect;
    private static NSImage barFill;
    private static NSRect barFillRect;
    private static NSImage barInd;
    private static NSRect barIndRect;

    static {
        barBorder = new NSImage(NSBundle.mainBundle().pathForResource("ProgressBarBorder", "tif"), false);
        barBorderRect = new NSRect(new NSPoint(0, 0), barBorder.size());
        barEmpty = new NSImage(NSBundle.mainBundle().pathForResource("ProgressBarEmpty", "tif"), false);
        barEmptyRect = new NSRect(new NSPoint(0, 0), barEmpty.size());
        barFill = new NSImage(NSBundle.mainBundle().pathForResource("ProgressBarFill", "tif"), false);
        barFillRect = new NSRect(new NSPoint(0, 0), barFill.size());
        barInd = new NSImage(NSBundle.mainBundle().pathForResource("ProgressBarInd", "tif"), false);
        barIndRect = new NSRect(new NSPoint(0, 0), barInd.size());
    }
    
    public PercentDisplay() {
        super();
    }

    public void drawInteriorWithFrameInView(NSRect cellFrame, NSView controlView) {
        float f = ((Number)objectValue()).floatValue();
        float h = barBorder.size().height();
        float y0 = (cellFrame.height() - h) / 2 + cellFrame.y();
        // We put the borders
        //barFill.drawInRect(new NSRect(new NSPoint(cellFrame.x(), y0), barBorder.size()), new NSRect(new NSPoint(0, 0), barInd.size()), NSImage.CompositeCopy, 1f);
        //barFill.drawInRect(new NSRect(new NSPoint(cellFrame.maxX()-1, y0), barBorder.size()), new NSRect(new NSPoint(0, 0), barInd.size()), NSImage.CompositeCopy, 1f);
        // We draw the interior
        if (f<=0f) {
            drawInRect(barInd, new NSRect(cellFrame.x()+1, y0, cellFrame.width()-2, h), barIndRect);
        } else if (f<100f) {
            float w = (cellFrame.width()-2)*f/100f;
            drawInRect(barFill, new NSRect(cellFrame.x()+1, y0, w, h), barFillRect);
            barEmpty.drawInRect(new NSRect(cellFrame.x()+w+1, y0, cellFrame.width()-w-2, h), barEmptyRect, NSImage.CompositeCopy, 1f);
        } else {
            drawInRect(barFill, new NSRect(cellFrame.x()+1, y0, cellFrame.width()-2, h), barFillRect);
        }
    }

    private static void drawInRect(NSImage img, NSRect r, NSRect imgRect) {
        float wImg = imgRect.width();
        float wRect = r.width();
        float x=0;
        for(; x<wRect-wImg; x+=wImg) {
            img.drawInRect(new NSRect(r.x()+x, r.y(), wImg, r.height()), imgRect, NSImage.CompositeCopy, 1f);
        }
        if (x<wRect)
            img.drawInRect(new NSRect(r.x()+x, r.y(), wRect-x, r.height()), new NSRect(new NSPoint(0, 0), new NSSize(wRect-x, imgRect.height())), NSImage.CompositeCopy, 1f);
    }
}

// $Log: PercentDisplay.java,v $
// Revision 1.2  2002/07/21 18:36:31  fortun
// Proper creator set
//
// Revision 1.1  2002/07/16 00:12:06  fortun
// *** empty log message ***
//
