//
//  Preferences.java
//  CocoDonkey
//  $Id: Preferences.java,v 1.6 2002/08/10 12:39:05 fortun Exp $
//
//  Created by Fred Bonzoun on Wed May 15 2002.
//  Copyright (c) 2002 Bonzoun. All rights reserved.
//
//  This library is free software; you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published
//  by the Free Software Foundation; either version 2.1 of the License, or
//  (at your option) any later version.
//
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

import com.apple.cocoa.application.*;
import com.apple.cocoa.foundation.*;

public class Preferences {

    static {
        NSMutableDictionary dico = new NSMutableDictionary();
        dico.setObjectForKey("~/mldonkey-distrib", "daemonLocation");
        dico.setObjectForKey(new Integer(10), "maxRecentConnections");
        dico.setObjectForKey(new Integer(5), "refreshInterval");
        dico.setObjectForKey(Boolean.FALSE, "autoConnectDaemon");
        dico.setObjectForKey(Boolean.FALSE, "autoLaunchDaemon");
        dico.setObjectForKey(Boolean.TRUE, "rememberWindowPos");
        dico.setObjectForKey(Boolean.FALSE, "autoStopDaemon");
        dico.setObjectForKey(Boolean.FALSE, "convertAvi");
        NSUserDefaults.standardUserDefaults().registerDefaults(dico);
    }

    public static String daemonLocation() {
        return NSUserDefaults.standardUserDefaults().stringForKey("daemonLocation");
    }

    public static void setDaemonLocation(String v) {
        NSUserDefaults.standardUserDefaults().setObjectForKey(v, "daemonLocation");
    }

    public static int refreshInterval() {
        return NSUserDefaults.standardUserDefaults().integerForKey("refreshInterval");
    }

    public static void setRefreshInterval(int v) {
        NSUserDefaults.standardUserDefaults().setIntegerForKey(v, "refreshInterval");
    }

    public static int maxRecentConnections() {
        return NSUserDefaults.standardUserDefaults().integerForKey("maxRecentConnections");
    }

    public static void setMaxRecentConnections(int v) {
        NSUserDefaults.standardUserDefaults().setIntegerForKey(v, "maxRecentConnections");
    }
    
    public static NSArray recentConnections() {
        NSArray liste = NSUserDefaults.standardUserDefaults().arrayForKey("recentConnections");
        if (liste==null)
            liste = new NSArray();
        return liste;
    }

    public static void addRecentConnection(NSDictionary conn) {
        NSMutableArray list = new NSMutableArray(recentConnections());
        // We check if the value is not already in the list, in that case we remove the old one
        for(int i=list.count()-1; i>=0; i--) {
            NSDictionary oldConn = (NSDictionary)list.objectAtIndex(i);
            java.util.Enumeration enumerator = conn.keyEnumerator();
            boolean areEqual = true;
            while (enumerator.hasMoreElements()) {
                Object key = enumerator.nextElement();
                if (!conn.objectForKey(key).equals(oldConn.objectForKey(key))) {
                    areEqual = false;
                    break;
                }
            }
            if (areEqual) {
                list.removeObjectAtIndex(i);
                break;
            }
        }
        // We add the new value at the beginning
        list.insertObjectAtIndex(conn, 0);
        while (list.count()>maxRecentConnections())
            list.removeLastObject();
        NSUserDefaults.standardUserDefaults().setObjectForKey(list, "recentConnections");
    }

    public static boolean autoConnectDaemon() {
        return NSUserDefaults.standardUserDefaults().booleanForKey("autoConnectDaemon");
    }

    public static void setAutoConnectDaemon(boolean b) {
        NSUserDefaults.standardUserDefaults().setBooleanForKey(b, "autoConnectDaemon");
    }

    public static boolean autoLaunchDaemon() {
        return NSUserDefaults.standardUserDefaults().booleanForKey("autoLaunchDaemon");
    }

    public static void setAutoLaunchDaemon(boolean b) {
        NSUserDefaults.standardUserDefaults().setBooleanForKey(b, "autoLaunchDaemon");
    }

    public static boolean rememberWindowPos() {
        return NSUserDefaults.standardUserDefaults().booleanForKey("rememberWindowPos");
    }

    public static void setRememberWindowPos(boolean b) {
        NSUserDefaults.standardUserDefaults().setBooleanForKey(b, "rememberWindowPos");
    }

    public static boolean autoStopDaemon() {
        return NSUserDefaults.standardUserDefaults().booleanForKey("autoStopDaemon");
    }

    public static void setAutoStopDaemon(boolean b) {
        NSUserDefaults.standardUserDefaults().setBooleanForKey(b, "autoStopDaemon");
    }

    public static void restoreWindowSize(NSWindowController wc) {
        String name = wc.getClass().getName() + "/size";
        String r = (String)NSUserDefaults.standardUserDefaults().objectForKey(name);
        if (r!=null && wc.window()!=null)
            wc.window().setFrameFromString(r);
    }

    public static void rememberWindowSize(NSWindowController wc) {
        if (wc==null || wc.window()==null)
            return;
        String r = wc.window().stringWithSavedFrame();
        String name = wc.getClass().getName() + "/size";
        NSUserDefaults.standardUserDefaults().setObjectForKey(r, name);
    }

    public static void rememberColumnSizes(NSWindowController wc, NSTableView t) {
        NSMutableArray liste = new NSMutableArray();
        for(int i=0; i<t.tableColumns().count(); i++) {
            NSTableColumn col = (NSTableColumn)t.tableColumns().objectAtIndex(i);
            Object name = col.identifier()!=null ? col.identifier() : "";
            NSArray value = new NSArray(new Object[]{name, new Integer(i), new Float(col.width())});
            liste.addObject(value);
        }
        String id = wc.getClass().getName() + "/" + t.tag();
        NSUserDefaults.standardUserDefaults().setObjectForKey(liste, id + ".cols");
    }

    public static void restoreColumnSizes(NSWindowController wc, NSTableView t) {
        if (t.window()==null)
            return;
        String id = wc.getClass().getName() + "/" + t.tag();
        NSArray liste = (NSArray)NSUserDefaults.standardUserDefaults().objectForKey(id + ".cols");
        if (liste!=null) {
            for(int i=0; i<liste.count(); i++) {
                NSArray value = (NSArray)liste.objectAtIndex(i);
                Object identifier = value.objectAtIndex(0);
                if (!identifier.equals("")) {
                    Number pos = (Number)value.objectAtIndex(1);
                    Number width = (Number)value.objectAtIndex(2);
                    NSTableColumn col = t.tableColumnWithIdentifier(identifier);
                    if (col!=null) {
                        if (width.floatValue()>5 && width.floatValue()<999)
                            col.setWidth(width.floatValue());
                        t.moveColumnToColumn(t.columnWithIdentifier(identifier), pos.intValue());
                    }
                }
            }
        }
    }

    public static boolean convertAvi() {
        return NSUserDefaults.standardUserDefaults().booleanForKey("convertAvi");
    }

    public static void setConvertAvi(boolean b) {
        NSUserDefaults.standardUserDefaults().setBooleanForKey(b, "convertAvi");
    }

    public static boolean trace() {
        return NSUserDefaults.standardUserDefaults().booleanForKey("trace");
    }

    public static void setTrace(boolean b) {
        NSUserDefaults.standardUserDefaults().setBooleanForKey(b, "trace");
    }
}

// $Log: Preferences.java,v $
// Revision 1.6  2002/08/10 12:39:05  fortun
// mldonkey -> mldonkey-distrib
//
// Revision 1.5  2002/07/07 01:22:17  fortun
// Little potential bug solved
//
// Revision 1.4  2002/07/04 23:56:03  fortun
// New trace preference
//
// Revision 1.3  2002/06/30 21:06:56  fortun
// New options for window pos & size, column pos & size, and others
//
// Revision 1.2  2002/06/02 17:53:14  fortun
// New functions to handle the "Recent Connections" menu
//
// Revision 1.1.1.1  2002/05/21 21:26:10  fortun
//
//
