//
//  PreferencesPanelController.java
//  CocoDonkey
//  $Id: PreferencesPanelController.java,v 1.3 2002/07/04 23:55:25 fortun Exp $
//
//  Created by Fred Bonzoun on Sun May 12 2002.
//  Copyright (c) 2002 Bonzoun. All rights reserved.
//
//  This library is free software; you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published
//  by the Free Software Foundation; either version 2.1 of the License, or
//  (at your option) any later version.
//
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

import com.apple.cocoa.foundation.*;
import com.apple.cocoa.application.*;

public class PreferencesPanelController extends WindowController {

    NSTextField daemonLocationField; /* IBOutlet */
    NSTextField refreshIntervalField; /* IBOutlet */
    NSMatrix autoConnectMatrix; /* IBOutlet */
    NSMatrix autoLaunchMatrix; /* IBOutlet */
    NSMatrix rememberPositionMatrix; /* IBOutlet */
    NSMatrix autoStopMatrix; /* IBOutlet */
    NSMatrix convertAviMatrix; /* IBOutlet */
    NSMatrix traceMatrix; /* IBOutlet */

    public PreferencesPanelController() {
        super("PreferencesPanel");
    }

    public void windowDidLoad() {
        super.windowDidLoad();
        daemonLocationField.setStringValue(Preferences.daemonLocation());
        refreshIntervalField.setIntValue(Preferences.refreshInterval());
        autoConnectMatrix.selectCellWithTag(Preferences.autoConnectDaemon() ? 1 : 0);
        autoLaunchMatrix.selectCellWithTag(Preferences.autoLaunchDaemon() ? 1 : 0);
        rememberPositionMatrix.selectCellWithTag(Preferences.rememberWindowPos() ? 1 : 0);
        autoStopMatrix.selectCellWithTag(Preferences.autoStopDaemon() ? 1 : 0);
        convertAviMatrix.selectCellWithTag(Preferences.convertAvi() ? 1 : 0);
        traceMatrix.selectCellWithTag(Preferences.trace() ? 1 : 0);
        showWindow(window());
    }

    protected boolean shouldRememberWindowPos() {
        return false;
    }
    
    public void browseDaemonLocation(Object sender) {
        NSOpenPanel panel = NSOpenPanel.openPanel();
        panel.setCanChooseDirectories(true);
        panel.setCanChooseFiles(false);
        if (panel.runModalInDirectory(daemonLocationField.stringValue(), null, new NSArray()) == NSOpenPanel.OKButton) {
            daemonLocationField.setStringValue((String)panel.filenames().objectAtIndex(0));
        }
    }
    
    public void cancel(Object sender) {
        NSApplication.sharedApplication().stopModalWithCode(0);
        window().close();
    }

    public void save(Object sender) {
        Preferences.setDaemonLocation(daemonLocationField.stringValue());
        Preferences.setRefreshInterval(refreshIntervalField.intValue());
        Preferences.setAutoConnectDaemon(autoConnectMatrix.selectedCell()!=null && autoConnectMatrix.selectedCell().tag()==1);
        Preferences.setAutoLaunchDaemon(autoLaunchMatrix.selectedCell()!=null && autoLaunchMatrix.selectedCell().tag()==1);
        Preferences.setRememberWindowPos(rememberPositionMatrix.selectedCell()!=null && rememberPositionMatrix.selectedCell().tag()==1);
        Preferences.setAutoStopDaemon(autoStopMatrix.selectedCell()!=null && autoStopMatrix.selectedCell().tag()==1);
        Preferences.setConvertAvi(convertAviMatrix.selectedCell()!=null && convertAviMatrix.selectedCell().tag()==1);
        Preferences.setTrace(traceMatrix.selectedCell()!=null && traceMatrix.selectedCell().tag()==1);
        
        NSApplication.sharedApplication().stopModalWithCode(1);
        window().close();
    }
}

// $Log: PreferencesPanelController.java,v $
// Revision 1.3  2002/07/04 23:55:25  fortun
// New trace preference
//
// Revision 1.2  2002/06/30 21:04:36  fortun
// Many new options
//
// Revision 1.1.1.1  2002/05/21 21:26:10  fortun
//
//
