//
//  PreviewWindowController.java
//  CocoDonkey
//  $Id: PreviewWindowController.java,v 1.4 2002/07/08 21:46:11 fortun Exp $
//
//  Created by Fred Bonzoun on Sun May 12 2002.
//  Copyright (c) 2002 Bonzoun. All rights reserved.
//
//  This library is free software; you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published
//  by the Free Software Foundation; either version 2.1 of the License, or
//  (at your option) any later version.
//
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

import com.apple.cocoa.foundation.*;
import com.apple.cocoa.application.*;

import java.io.*;
import net.bonzoun.cocodonkey.*;

public class PreviewWindowController extends WindowController {

    NSTextField nameField; /* IBOutlet */
    NSProgressIndicator progressBar; /* IBOutlet */
    NSTextField statusField; /* IBOutlet */
    NSButton previewButton; /* IBOutlet */

    private DownloadItem item;
    private File source;
    private File dest;
    private DonkeyTalker talker;
    private Worker worker;
    private boolean useMPlayer;
    
    public PreviewWindowController(DownloadItem item, DonkeyTalker talker, boolean mplayer) {
        super("PreviewWindow");
        this.item = item;
        this.talker = talker;
        this.useMPlayer = mplayer;
    }

    public void windowDidLoad() {
        super.windowDidLoad();
        nameField.setStringValue(item.name());
        statusField.setStringValue(Messages.string("Transfering..."));

        talker.pause(item);
        // We get the location of the file
        File dir = new File(Preferences.daemonLocation());
        if (item.isFinished())
            dir = new File(dir, "incoming");
        else
            dir = new File(dir, "temp");
        if (!dir.exists()) {
            window().close();
            return;
        }
        source = new File(dir, item.hash());
        if (!source.exists()) {
            window().close();
            return;
        }
        if (useMPlayer) {
            previewButton.setEnabled(true);
            preview(this);
        } else {
            // We convert the name of the dest file
            String dname = item.name();
            dest = new File(Preferences.daemonLocation() + "/" + dname);
            if (dest.exists())
                dest.delete();
            // We transfert it
            new Transfert().start();
            
            progressBar.setIndeterminate(false);
            progressBar.setDoubleValue(0);
            progressBar.setMaxValue(source.length());
    
            worker = new Worker();
            worker.start();
        }
    }
    
    public void cancel(Object sender) { // IBAction
        window().close();
    }

    public void preview(Object sender) { // IBAction
        if (useMPlayer) {
            try {
                Runtime.getRuntime().exec(new String[]{"osascript", "-e", "tell application \"Terminal\"\nactivate\ndo script with command \"/usr/local/bin/mplayer -vo sdl -ao sdl -framedrop -subpos 85 -noautosub  -idx '" + Preferences.daemonLocation() + "/temp/" + item.hash() + "'\"\nend tell"});
                statusField.setStringValue(Messages.string("Done"));
            }
            catch (IOException e) {
                statusField.setStringValue(e.getMessage());
            }
            progressBar.setIndeterminate(false);
            progressBar.setDoubleValue(100);
            progressBar.setMaxValue(100);
        } else {
            try {
                Runtime.getRuntime().exec(new String[]{"open", dest.getAbsolutePath()});
            }
            catch(IOException e) {
                statusField.setStringValue(e.getMessage());
            }
        }
    }
    
    private void clean() {
        if (worker!=null) {
            worker.stopRunning();
            try {
                Thread.sleep(200);
            } catch (InterruptedException e) {}
        }
        if (dest!=null)
            dest.delete();
        talker.resume(item);
    }

    public void windowWillClose(NSNotification aNotification) {
        super.windowWillClose(aNotification);
        clean();
    }
    
    private class Transfert extends WorkerThread {
        public void run() {
            // We transfert the file
            FileInputStream in = null;
            FileOutputStream out = null;
            try {
                boolean complete = false;
                in = new FileInputStream(source);
                out = new FileOutputStream(dest);
                byte[] buffer = new byte[4096];
                while(worker==null || worker.isRunning()) {
                    int n = in.read(buffer);
                    if (n<0) {
                        complete = true;
                        break;
                    }
                    out.write(buffer, 0, n);
                }
                progressBar.setDoubleValue(dest.length());
                if (complete) {
                    previewButton.setEnabled(true);
                    statusField.setStringValue(Messages.string("Copied"));
                    preview(null);
                }
            }
            catch(IOException e) {
                statusField.setStringValue(e.getMessage());
            }
            finally {
                if (worker!=null)
                    worker.stopRunning();
                try {
                    if (in!=null)
                        in.close();
                    if (out!=null)
                        out.close();
                    talker.resume(item);
                }
                catch(IOException e) {
                    e.printStackTrace();
                }
            }
        }
    }
    
    private class Worker extends WorkerThread {
        protected boolean work() throws Exception {
            progressBar.setDoubleValue(dest.length());
            return true;
        }

        protected void dies() {
            worker = null;
        }
    }
}

// $Log: PreviewWindowController.java,v $
// Revision 1.4  2002/07/08 21:46:11  fortun
// If mplayer, preview is done with an Applescript through Terminal (so that the keyboard is active)
//
// Revision 1.3  2002/07/07 01:20:42  fortun
// Uses mplayer for the preview
//
// Revision 1.2  2002/06/01 15:15:02  fortun
// Better stop of the WorkerThread
//
// Revision 1.1.1.1  2002/05/21 21:26:10  fortun
//
//
