//
//  SearchedItemSource.java
//  CocoDonkey
//  $Id: SearchedItemSource.java,v 1.8 2003/01/01 17:35:37 fortun Exp $
//
//  Created by Fred Bonzoun on Sun May 12 2002.
//  Copyright (c) 2002 Bonzoun. All rights reserved.
//
//  This library is free software; you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published
//  by the Free Software Foundation; either version 2.1 of the License, or
//  (at your option) any later version.
//
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

import com.apple.cocoa.foundation.*;
import com.apple.cocoa.application.*;

import net.bonzoun.cocodonkey.*;
import java.util.*;

public class SearchedItemSource {

    NSTableView tableView;
    TableViewSorter tableSorter;
    DownloadItemSource downloadItemSource;

    public void setDownloadItemSource(DownloadItemSource downloadItemSource) {
        this.downloadItemSource = downloadItemSource;
    }

    public void windowDidLoad() {
        tableSorter.addSortedColumn("Name", new GenericItem.CompareName());
        tableSorter.addSortedColumn("Size", new GenericItem.CompareSize());
        tableSorter.addSortedColumn("Clients", new GenericItem.CompareNbClients());
    }
    
    /*package*/ SearchedItem itemAtRow(int row) {
        if (row>=tableSorter.list().size())
            return null;
        return (SearchedItem)tableSorter.list().get(row);
    }

    /*package*/ int itemsFound() {
        return tableSorter.list().size();
    }

    /*package*/ void setElements(List list) {
        if (list==null)
            list = new ArrayList();
        if (list.size()!=tableSorter.list().size()) {
            tableSorter.setList(list);
            //tableView.reloadData(); // Done in the TableSorter
        } else {
            if (!GenericItem.sameList(list, tableSorter.list())) {
                tableSorter.setList(list);
                //tableView.setNeedsDisplay(true); // Done in the TableSorter
            }
        }
    }

    public int numberOfRowsInTableView(NSTableView aTableView) {
        if (tableSorter!=null)
            return tableSorter.list().size();
        else
            return 0;
    }
    
    public final Object tableViewObjectValueForLocation(NSTableView aTableView, NSTableColumn aTableColumn, int rowIndex) {
        if (rowIndex>=tableSorter.list().size())
            return null;
        SearchedItem item = (SearchedItem)tableSorter.list().get(rowIndex);
        String name = (String)aTableColumn.identifier();
        Object res = "?";
        if (name.equals("Name"))
            res = item.name();
        else if (name.equals("Size"))
            res = item.prettySize();
        else if (name.equals("Length")) {
            if (item.length()!=null)
                res = item.length();
            else
                res = "";
        }
        else if (name.equals("Type")) {
            if (item.type()!=null) {
                if (item.extension()!=null)
                    res = item.type() + ": " + item.extension();
                else
                    res = item.type();
            } else {
                res = "";
            }
        }
        else if (name.equals("Codec"))
            res = item.codec();
        else if (name.equals("Clients")) {
            if (item.nbClients()>0)
                res = String.valueOf(item.nbClients());
            else
                res = "";
        } else if (name.equals("Status")) {
            DownloadItem di = downloadItemSource.item(item.size(), item.hash());
            if (di!=null) {
                res = di.isFinished() ? "V" : "!";
            } else {
                List aliases = item.aliases();
                if (aliases.size()>0 && ((String)aliases.get(aliases.size()-1)).indexOf("ALREADY DOWNLOADED")>=0) {
                    res = "v";
                } else {
                    res = "";
                }
            }
        }
        return res;
    }

    public void tableViewDidClickTableColumn(NSTableView tableView, NSTableColumn tableColumn) {
        tableSorter.tableViewDidClickTableColumn(tableView, tableColumn);
    }
}

// $Log: SearchedItemSource.java,v $
// Revision 1.8  2003/01/01 17:35:37  fortun
// No display of the clients if their are none
//
// Revision 1.7  2002/07/25 23:33:30  fortun
// Now the display updates are done in the TableSorter
//
// Revision 1.6  2002/07/22 00:30:23  fortun
// NSArrays are now Lists
//
// Revision 1.5  2002/07/07 20:07:28  fortun
// Simpler handling of the selection of the header
//
// Revision 1.4  2002/07/07 19:46:31  fortun
// Now handles the selection of the header
//
// Revision 1.3  2002/06/29 17:55:45  fortun
// Now displays a status
//
// Revision 1.2  2002/05/27 22:03:54  fortun
// Uses the new TableViewSorter to sort the list
// Optimized to only redraw when needed
//
// Revision 1.1.1.1  2002/05/21 21:26:10  fortun
//
//
