//
//  SortedtableHeader.java
//  CocoDonkey
//  $Id: SortedTableHeaderCell.java,v 1.5 2002/07/09 22:08:08 fortun Exp $
//
//  Created by Fred Bonzoun on Mon May 27 2002.
//  Copyright (c) 2002 Bonzoun. All rights reserved.
//
//  This library is free software; you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published
//  by the Free Software Foundation; either version 2.1 of the License, or
//  (at your option) any later version.
//
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

import com.apple.cocoa.foundation.*;
import com.apple.cocoa.application.*;


public class SortedTableHeaderCell extends NSTableHeaderCell {

    private static final float marging = 2;

    private static NSImage headerUnsortedImage;
    private static NSRect headerUnsortedRect;
    private static NSImage headerUnsortedSepImage;
    private static NSImage headerSortedImage;
    private static NSRect headerSortedRect;
    private static NSImage headerSortedSepImage;
    
    private int sortOrdering;
    //private boolean isSelected;

    public SortedTableHeaderCell(String aString) {
        super(aString);
        if (headerUnsortedImage==null) {
            String filename = NSBundle.mainBundle().pathForResource("headerUnsorted", "tif");
            headerUnsortedImage = new NSImage(filename, false);
            headerUnsortedRect = new NSRect(new NSPoint(0f, 0f), headerUnsortedImage.size());
            filename = NSBundle.mainBundle().pathForResource("headerUnsortedSep", "tif");
            headerUnsortedSepImage = new NSImage(filename, false);
            filename = NSBundle.mainBundle().pathForResource("headerSorted", "tif");
            headerSortedImage = new NSImage(filename, false);
            headerSortedRect = new NSRect(new NSPoint(0f, 0f), headerSortedImage.size());
            filename = NSBundle.mainBundle().pathForResource("headerSortedSep", "tif");
            headerSortedSepImage = new NSImage(filename, false);
        }
    }

    public void setOrdering(int sort) {
        sortOrdering = sort;
    }
    
    public void drawInteriorWithFrameInView(NSRect cellFrame, NSView controlView) {
        // super.drawInteriorWithFrameInView(cellFrame, controlView);
        // We draw the background
        if (sortOrdering!=0 || state()!=OffState) {
            headerSortedImage.drawInRect(cellFrame, headerSortedRect, NSImage.CompositeCopy, 1f);
            headerSortedSepImage.compositeToPoint(new NSPoint(cellFrame.maxX()-1, cellFrame.maxY()), NSImage.CompositeCopy);
        } else {
            headerUnsortedImage.drawInRect(cellFrame, headerUnsortedRect, NSImage.CompositeCopy, 1f);
            headerUnsortedSepImage.compositeToPoint(new NSPoint(cellFrame.maxX()-1, cellFrame.maxY()), NSImage.CompositeCopy);
        }
        // We prepare some sizes
        float x0 = cellFrame.maxX() - cellFrame.height()/2 - marging;
        float y0 = cellFrame.y() + cellFrame.height()/4;
        float w = cellFrame.height()/2;
        float h = cellFrame.height()/2;
        // We draw the text
        float xr = cellFrame.x();
        if (alignment()==NSText.LeftTextAlignment || alignment()==NSText.JustifiedTextAlignment)
            xr += 3f;
        if (x0 > cellFrame.x()) {
            NSRect rText = new NSRect(new NSPoint(xr, cellFrame.y()), new NSSize(x0 - cellFrame.x(), cellFrame.size().height()));
            font().set();
            NSGraphics.drawAttributedString(attributedStringValue(), rText);
        }
        // We draw the little triangle for the sorting
        if (sortOrdering!=0 || state()!=OffState)
            NSColor.darkGrayColor().set();
        else
            NSColor.grayColor().set();
        if (sortOrdering>0) {
            NSBezierPath path = new NSBezierPath();
            path.moveToPoint(new NSPoint(x0 + w/2,	y0));
            path.lineToPoint(new NSPoint(x0,		y0 + h));
            path.lineToPoint(new NSPoint(x0 + w,	y0 + h));
            path.lineToPoint(new NSPoint(x0 + w/2,	y0));
            path.fill();
        }
        else if (sortOrdering<0) {
            NSBezierPath path = new NSBezierPath();
            path.moveToPoint(new NSPoint(x0 + w/2,	y0 + h));
            path.lineToPoint(new NSPoint(x0 + w,	y0));
            path.lineToPoint(new NSPoint(x0,		y0));
            path.lineToPoint(new NSPoint(x0 + w/2,	y0 + h));
            path.fill();
        }
        else {
            NSBezierPath.fillRect(new NSRect(x0, y0, w, h));
        }
    }
}

// $Log: SortedTableHeaderCell.java,v $
// Revision 1.5  2002/07/09 22:08:08  fortun
// Was failing when resizing
//
// Revision 1.4  2002/07/08 21:43:09  fortun
// Space on the left for the text if left aligned
//
// Revision 1.3  2002/07/07 20:07:28  fortun
// Simpler handling of the selection of the header
//
// Revision 1.2  2002/07/07 19:35:12  fortun
// Now displays the selection of the header
//
// Revision 1.1  2002/05/27 22:05:44  fortun
// Draws a header with a visible triangle for the sort order
//
