//
//  WindowController.java
//  CocoDonkey
//  $Id: WindowController.java,v 1.7 2002/07/30 00:16:01 fortun Exp $
//
//  Created by Fred Bonzoun on Sun May 19 2002.
//  Copyright (c) 2002 Bonzoun. All rights reserved.
//
//  This library is free software; you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published
//  by the Free Software Foundation; either version 2.1 of the License, or
//  (at your option) any later version.
//
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

import com.apple.cocoa.foundation.*;
import com.apple.cocoa.application.*;

/***
 * All WindowController descendants must be delegates of their window.
 ***/
public class WindowController extends NSWindowController {

    /***
     * Keeps a list of the opened controllers.
     * Required because otherwise the controller will be freed by the gc
     ***/
    private static NSMutableArray controllers = new NSMutableArray();
    private boolean shouldRememberWindowPos;

    protected WindowController(String name) {
        this(name, true);
    }

    protected NSScrollView scrollViewOf(NSView view) {
        if (view instanceof NSScrollView)
            return (NSScrollView)view;
        else if (view.superview()!=null)
            return scrollViewOf(view.superview());
        else
            return null;
    }
    
    protected WindowController(String name, boolean value) {
        super(name);
        shouldRememberWindowPos = value;
    }
    
    protected boolean shouldRememberWindowPos() {
        return shouldRememberWindowPos;
    }
    
    public void windowDidLoad() {
        controllers.addObject(this);
        if (shouldRememberWindowPos() && Preferences.rememberWindowPos())
            Preferences.restoreWindowSize(this);
    }

    public void windowWillClose(NSNotification aNotification) {
        controllers.removeObject(this);
        if (shouldRememberWindowPos())
            Preferences.rememberWindowSize(this);
    }

    protected NSColor colorForRow(NSTableView aTableView, int rowIndex) {
        if (rowIndex%2 == 0) {
            return null;
        } else {
            return NSColor.colorWithDeviceRGB(0.95f, 0.95f, 1.0f, 1.0f);
        }
    }
    
    private int oldRow = -1;

    public void tableViewWillDisplayCell(NSTableView aTableView, Object aCell, NSTableColumn aTableColumn, int rowIndex) {
        if (oldRow!=rowIndex && rowIndex!=aTableView.selectedRow()) {
            oldRow=rowIndex;
            NSColor col = colorForRow(aTableView, rowIndex);
            if (col!=null) {
                col.set();
                NSBezierPath.fillRect(aTableView.rectOfRow(rowIndex));
            }
        }
        if (aCell instanceof NSTextFieldCell)
            ((NSTextFieldCell)aCell).setDrawsBackground(false);
    }
}

// $Log: WindowController.java,v $
// Revision 1.7  2002/07/30 00:16:01  fortun
// Optimization: white rows background is not drawn anymore
//
// Revision 1.6  2002/07/21 18:37:32  fortun
// Implements colors for the rows
// New scrollViewOf() function
//
// Revision 1.5  2002/07/14 22:41:09  fortun
// Detail
//
// Revision 1.4  2002/07/05 22:47:21  uid85676
// New option for controllers that don't retain their size
//
// Revision 1.3  2002/07/04 23:57:39  fortun
// New option not to remember the window pos
//
// Revision 1.2  2002/06/30 21:07:37  fortun
// Automatically remembers window pos & size
//
// Revision 1.1.1.1  2002/05/21 21:26:10  fortun
//
//
