//
//  CamlFileParser.java
//  CocoDonkey
//	$Id: CamlData.java,v 1.1 2002/08/21 21:51:31 fortun Exp $
//
//  Created by Fred Bonzoun on Tue Aug 20 2002.
//  Copyright (c) 2002 Bonzoun. All rights reserved.
//
//  This library is free software; you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published
//  by the Free Software Foundation; either version 2.1 of the License, or
//  (at your option) any later version.
//
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

package net.bonzoun.cocodonkey;

/***
 * Class that encpsulates data that come from a file saved by Caml
 ***/
public abstract class CamlData {

    private static final CamlData empty = new CamlData.Data("");
    
    public String getString() {
        return toString();
    }

    public int size() {
        return -1;
    }
    
    public int getInt() {
        return parseInt(getString());
    }

    public double getDouble() {
        return parseDouble(getString());
    }

    public CamlData get(int i) {
        return empty;
    }
    
    public String getString(int i) {
        return get(i).toString();
    }

    public int getInt(int i) {
        return parseInt(getString(i));
    }

    public double getDouble(int i) {
        return parseDouble(getString(i));
    }

    public boolean getBoolean(int i) {
        return getString(i).equals("true");
    }
    
    public CamlData get(String key) {
        return empty;
    }
    
    public String getString(String key) {
        return get(key).toString();
    }

    public int getInt(String key) {
        return parseInt(getString(key));
    }

    public double getDouble(String key) {
        return parseDouble(getString(key));
    }

    public boolean getBoolean(String key) {
        return getString(key).equals("true");
    }
    
    private static int parseInt(String s) {
        try {
            return Integer.parseInt(s);
        }
        catch(NumberFormatException e) {
            return -1;
        }
    }

    private static double parseDouble(String s) {
        try {
            return Double.parseDouble(s);
        }
        catch(NumberFormatException e) {
            return -1.;
        }
    }

    public static class List extends CamlData {
        private java.util.List list;

        public List(java.util.List value) {
            list = value;
        }

        public int size() {
            return list.size();
        }

        public String toString() {
            return list.toString();
        }

        public CamlData get(int i) {
            if (i>=0 && i<list.size())
                return (CamlData)list.get(i);
            else
                return empty;
        }
    }

    public static class Map extends CamlData {
        private java.util.Map map;

        public Map(java.util.Map value) {
            map = value;
        }

        public int size() {
            return map.size();
        }

        public String toString() {
            return map.toString();
        }

        public CamlData get(String key) {
            Object value = map.get(key);
            if (value!=null)
                return (CamlData)value;
            else
                return empty;
        }
    }

    public static class Data extends CamlData {
        private String string;

        public Data(String value) {
            string = value;
        }

        public String toString() {
            return string;
        }
    }
}

// $Log: CamlData.java,v $
// Revision 1.1  2002/08/21 21:51:31  fortun
// *** empty log message ***
//
