//
//  DownloadItem.java
//  CocoDonkey
//
//  Created by Fred Bonzoun on Sat May 11 2002.
//  Copyright (c) 2002 Bonzoun. All rights reserved.
//
//  This library is free software; you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published
//  by the Free Software Foundation; either version 2.1 of the License, or
//  (at your option) any later version.
//
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

package net.bonzoun.cocodonkey;

public class DownloadItem extends GenericItem {

    protected float percent;
    protected int downloaded;
    protected float rate;
    protected String chunks;
    protected int nbConn;
    protected RateCalculator rateCalculator = new RateCalculator();

    protected int completion;
    
    /*package*/ DownloadItem(int id, String name, float percent, int downloaded, int size, float rate) {
        super(id, name);
        this.percent = percent;
        this.downloaded = downloaded;
        this.size = size;
        this.rate = rate;
        completion = 0;
    }

    /*package*/ DownloadItem(int id, String name, int size, String hash) {
        super(id, name);
        this.size = size;
        this.hash = hash;
        this.percent = 100;
        completion = 1;
    }

    public boolean isFinished() {
        return completion==1;
    }
    
    public boolean updateWith(DownloadItem item) {
        boolean modified = false;
        if (completion!=1 && item.completion!=1) {
            /*if (id!=item.id) {
                modified = true;
                id=item.id;
            }
            if (name!=item.name) {
                modified = true;
                name=item.name;
            }*/
            if (percent!=item.percent) {
                modified = true;
                percent=item.percent;
            }
            if (downloaded!=item.downloaded) {
                modified = true;
                downloaded=item.downloaded;
            }
            /*if (size!=item.size) {
                modified = true;
                size=item.size;
            }*/
            if (rate!=item.rate) {
                modified = true;
                rate=item.rate;
            }
        }
        else if (completion!=1 && item.completion==1) {
            // We have got a finished item
            modified = true;
            if (completion==0) {
                name=item.name;
                hash=item.hash;
            }
            completion = 1;
        }
        return modified;
    }

    public void completeData(String name, int total, String hash, int downloaded) {
        if (completion!=1) {
            this.name=name;
            this.hash=hash;
            completion = 2;
        }
    }

    public float percent() {
        return percent;
    }

    public int downloaded() {
        if (isFinished())
            return size();
        else
            return downloaded;
    }

    public String prettyDownloaded() {
        return displaySize(downloaded());
    }
    
    public float rate() {
        return rate;
    }

    public int timeLeft() {
        rateCalculator.addRate(rate());
        float meanRate = rateCalculator.meanRate();
        
        if (meanRate > 0) {
            int remaining = size() - downloaded();                    
            return  (int) (Math.round((float) remaining / (1024 * meanRate) ));
        } else {
            return -1;
        }
    }

    public void setChunks(String chunks) {
        this.chunks = chunks;
    }
    
    public String chunks() {
        return chunks;
    }

    public float chunksFoundPercent() {
        return chunksPercentOf('V') + chunksPercentOf('1') + chunksPercentOf('p') + chunksPercentOf('!') + chunksPercentOf('?');
    }

    public float chunksNotFoundPercent() {
        return chunksPercentOf('_') + chunksPercentOf('0') + chunksPercentOf('.');
    }
    
    private float chunksPercentOf(char t) {
        if (chunks==null || chunks.length()==0)
            return 0;
        int n = 0, l = chunks.length();
        for(int i=0; i<l; i++) {
            char c = chunks.charAt(i);
            if (c==t)
                n++;
        }
        return (float)100*n/l;
    }
    
    public void setNbConn(int n) {
        nbConn = n;
    }

    public int nbConn() {
        return nbConn;
    }
    
    public static class ComparePercent extends MyComparator {
        public final int compare(Object o1, Object o2) {
            DownloadItem item1 = (DownloadItem)o1;
            DownloadItem item2 = (DownloadItem)o2;
            if (item1.percent()<item2.percent())
                return -1;
            else if (item1.percent()==item2.percent())
                return 0;
            else
                return 1;
        }
    }

    public static class CompareRate extends MyComparator {
        public final int compare(Object o1, Object o2) {
            DownloadItem item1 = (DownloadItem)o1;
            DownloadItem item2 = (DownloadItem)o2;
            if (item1.rate()<item2.rate())
                return -1;
            else if (item1.rate()==item2.rate())
                return 0;
            else
                return 1;
        }
    }
    
    public static class CompareDownloaded extends MyComparator {
        public final int compare(Object o1, Object o2) {
            DownloadItem item1 = (DownloadItem)o1;
            DownloadItem item2 = (DownloadItem)o2;
            if (item1.downloaded()<item2.downloaded())
                return -1;
            else if (item1.downloaded()==item2.downloaded())
                return 0;
            else
                return 1;
        }
    }
}

// $Log: DownloadItem.java,v $
// Revision 1.7  2003/01/01 19:56:54  fortun
// Bug in the % of chunks found
//
// Revision 1.6  2002/06/29 11:22:52  fortun
// Handles the cached chunk states
//
// Revision 1.5  2002/06/09 15:43:01  fortun
// Added chunksFoundPercent() method
// Uses the RateCalculator
//
// Revision 1.4  2002/06/08 12:29:18  fortun
// Handles finished items
//
// Revision 1.3  2002/06/08 12:20:07  fortun
// Estimation of the time remaining
//
// Revision 1.2  2002/05/27 22:09:40  fortun
// Added some comparators (for sorting)
//
// Revision 1.1.1.1  2002/05/21 21:26:13  fortun
//
//
