//
//  SearchParser.java
//  CocoDonkey
//	$Id: SearchParser.java,v 1.2 2002/08/21 21:48:44 fortun Exp $
//
//  Created by Fred Bonzoun on Sun Jul 21 2002.
//  Copyright (c) 2002 Bonzoun. All rights reserved.
//
//  This library is free software; you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published
//  by the Free Software Foundation; either version 2.1 of the License, or
//  (at your option) any later version.
//
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

package net.bonzoun.cocodonkey;

import java.util.*;

public class SearchParser extends StringParser {

    private HashMap leftHandSides = new HashMap();
    private int leftToBeDone;
    private int searchNumber;

    public SearchParser(String s) {
        super(s);
        leftToBeDone = getLastInt(0);
        getNextChar();
        searchNumber = getNextIntUntil("]", -1);
        getNextChar();
        parse();
        //System.out.println("##" + s + "##");
        //System.out.println("## [" + searchNumber + "] " + toStringExtended() + " {" + leftToBeDone + "} ##");
    }

    public int searchNumber() {
        return searchNumber;
    }

    public int leftToBeDone() {
        return leftToBeDone;
    }

    public boolean isExtended() {
        if (leftHandSides.size()==0)
            return false;
        return leftHandSides.size()>1 || !leftHandSides.containsKey("");
    }

    /** Returns the value for the given option, "" if the option is not set **/
    public String stringFor(String key) {
        List l = (List)leftHandSides.get(key.toLowerCase());
        if (l!=null) {
            StringBuffer out = new StringBuffer();
            Iterator datas = l.iterator();
            boolean isFirst = true;
            while(datas.hasNext()) {
                BinOperator op = (BinOperator)datas.next();
                if (isFirst)
                    isFirst = false;
                else
                    out.insert(0, ' ');
                out.insert(0, op.rightHandSide);
            }
            return out.toString();
        } else {
            return "";
        }
    }

    public String stringFor(String key, String sop) {
        List l = (List)leftHandSides.get(key.toLowerCase());
        if (l!=null) {
            Iterator datas = l.iterator();
            while(datas.hasNext()) {
                BinOperator op = (BinOperator)datas.next();
                if (op.op.equals(sop))
                    return op.rightHandSide;
            }
        }
        return "";
    }
    
    public String toString() {
        return stringFor("");
    }

    public String toStringExtended() {
        StringBuffer out = new StringBuffer();
        out.append(stringFor(""));
        // We parse all the keys
        boolean isFirst = true;
        Iterator keys = leftHandSides.keySet().iterator();
        while(keys.hasNext()) {
            String lhs = (String)keys.next();
            String rhs = stringFor(lhs);
            if (lhs.length()>0 && rhs!=null && rhs.length()>0) {
                if (isFirst) {
                    isFirst = false;
                    out.append(" (");
                } else {
                    out.append(", ");
                }
                out.append(lhs);
                out.append(" =\"");
                out.append(rhs);
                out.append('\"');
            }
        }
        if (!isFirst)
            out.append(")");
        return out.toString();
    }

    /** Parses a search definition step, ie xxx AND xxx **/
    protected void parse() {
        // We get one data
        parseNextData();
        // We test for the end of the stream
        if (testNextChar()==')') {
            // One level less of parentheses. We must return
            getNextChar();
            return;
        }
        // We test for the operator
        String s = getNextString();
        if (s.equals("AND")) {
            // More data to get
            parse();
        } else {
            // No more data, or unknown form of data
        }
    }

    /** Parses a search data step, ie CONTAINS[xxx] or [xxx]CONTAINS[xxx] or [xxx]xx999 **/
    protected void parseNextData() {
        char c = testNextChar();
        if (c=='(') {
            // One more level of parentheses. Will return at the corresponding closing parentheses
            getNextChar();
            parse();
            return;
        }
        // We get the Left hand side
        String lhs;
        if (c=='[') {
            // We have to parse a String data
            getNextChar();
            lhs = getNextStringUntil("]");
            getNextChar();
        } else {
            // We have no lhs
            lhs = "";
        }
         // We get the operator and the Right Hand Side
        String op, rhs = null;
        if ("<>!=".indexOf(testNextChar())>=0) {
            // Compare with a number
            op = getNextStringUntil(INTEGER);
            if (testNextChar()!=0)
                rhs = getNextStringUntil(")" + WHITESPACE);
        } else {
            // CONTAINS a String
            op = getNextStringUntil("[");
            if (getNextChar()=='[') {
                rhs = getNextStringUntil("]");
                getNextChar();
            }
        }
        if (rhs!=null)
            addOperator(new BinOperator(lhs, op, rhs));
    }

    protected void addOperator(BinOperator binOp) {
        String key = binOp.leftHandSide.toLowerCase();
        List l = (List)leftHandSides.get(key);
        if (l==null) {
            // We must create a new Vector
            l = new ArrayList();
            leftHandSides.put(key, l);
        }
        l.add(binOp);
    }
    
    public class BinOperator {
        protected String op;
        protected String rightHandSide;
        protected String leftHandSide;

        protected BinOperator(String lhs, String op, String rhs) {
            this.op = op;
            this.rightHandSide = rhs;
            this.leftHandSide = lhs;
        }
    }
}

// $Log: SearchParser.java,v $
// Revision 1.2  2002/08/21 21:48:44  fortun
// NUMBER becomes INTEGER
//
// Revision 1.1  2002/07/21 18:40:00  fortun
// *** empty log message ***
//
