//
//  SshTunnel.java
//  CocoDonkey
//  $Id: SshTunnel.java,v 1.2 2002/07/05 21:34:26 uid85676 Exp $
//
//  Created by Fred Bonzoun on Thu May 23 2002.
//  Copyright (c) 2002 Bonzoun. All rights reserved.
//
//  This library is free software; you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published
//  by the Free Software Foundation; either version 2.1 of the License, or
//  (at your option) any later version.
//
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

package net.bonzoun.cocodonkey;

import java.io.*;

public class SshTunnel {

    private static int nextPort = 4001;
    
    private Process tunnel;
    private String hostname;
    private String ackAppPath;
    private int portIn;
    private int portOut;

    public SshTunnel(String hostname) throws IOException {
        this(hostname, nextPort++, 4000, null);
    }

    public SshTunnel(String hostname, String ackAppPath) throws IOException {
        this(hostname, nextPort++, 4000, ackAppPath);
    }
    
    public SshTunnel(String hostname, int portIn, int portOut, String ackAppPath) throws IOException {
        this.hostname = hostname;
        this.portIn = portIn;
        this.portOut = portOut;
        this.ackAppPath = ackAppPath;
        start();
    }
    
    private void start() throws IOException {
        String cmd = "/usr/bin/ssh -N -C -L " + portIn + ":127.0.0.1:" + portOut + " " + hostname;
        //String cmd = "/usr/bin/ssh";
        //String[] args = new String[]{"-N", "-C", "-L", "" + portIn + ":127.0.0.1:" + portOut, hostname};
        String[] env = null;
        //if (ackAppPath!=null)
        //    env = new String[]{"DISPLAY=localhost", "SSH_ACKPASS="+ackAppPath};
        //run(cmd, args, env);
        tunnel = Runtime.getRuntime().exec(cmd, env);
        try {
            Thread.sleep(5000);
            StringWriter res = new StringWriter();
            //while(tunnel.getInputStream().available()>0)
            //    System.out.print((char)tunnel.getInputStream().read());
            while(tunnel.getErrorStream().available()>0)
                res.write((char)tunnel.getErrorStream().read());
            tunnel.exitValue();
            String msg = res.toString();
            if (msg.indexOf("Host key verification failed")>=0)
                msg = "You must connect to this host with the terminal at least once to validate its key";
            else if (msg.indexOf("Permission denied")>=0)
                msg = "Password requested, you must install your ssh public identity file on the remote host";
            throw new IOException(msg);
        }
        catch(InterruptedException e) {
        }
        catch(IllegalThreadStateException e) {
            // good !
        }
            
    }

    public int port() {
        return portIn;
    }
    
    public void close() {
        tunnel.destroy();
    }

    public void restart() throws IOException {
        close();
        start();
    }
}

// $Log: SshTunnel.java,v $
// Revision 1.2  2002/07/05 21:34:26  uid85676
// No more native launch and kill
//
// Revision 1.1  2002/05/25 23:41:40  fortun
// *** empty log message ***
//
